using EVCB_OCPP.DBAPI;
using EVCB_OCPP.DBAPI.Jobs;
using EVCB_OCPP.DBAPI.Services;
using EVCB_OCPP.DBAPI.Services.DbService;
using EVCB_OCPP.DBAPI.Services.ServerMessageServices;
using EVCB_OCPP.Domain.Extensions;
using NLog.Extensions.Logging;
using NLog.Web;
using ServiceStack.Redis;
using SQLitePCL;

raw.SetProvider(new SQLite3Provider_e_sqlite3());

var builder = Host.CreateDefaultBuilder(args)
    .ConfigureLogging((context, loggingBuilder) => {
        loggingBuilder.ClearProviders();
        NLog.LogManager.Configuration = new NLogLoggingConfiguration(context.Configuration.GetSection("NLog"));
    })
    .UseNLog()
    .ConfigureWebHostDefaults(webBuilder =>
    {
        webBuilder.UseStartup<Startup>();
        webBuilder.ConfigureKestrel(serverOptions =>
        {
            serverOptions.Limits.MaxRequestBodySize = null;
        });
    })
    .ConfigureServices((context, services) => {
        services.AddSingleton<IRedisClientsManager>(new RedisManagerPool(context.Configuration["RedisConnectionString"]));

        services.AddJobs();

        services.AddMainDbContext(context.Configuration);
        services.AddTransient<IMainDbService, MainDbService>();

        services.AddTransient<IServerMessageService, SourceDbServerMessageService>();
        services.AddMemDbService();
    });

var app = builder.Build();
await app.RunAsync();